diff --git a/src/tools/PerfectHash.pm b/src/tools/PerfectHash.pm
index 74fb1f2ef6..4a029621b2 100644
--- a/src/tools/PerfectHash.pm
+++ b/src/tools/PerfectHash.pm
@@ -78,18 +78,25 @@ sub generate_hash_function
 
 	# Try different hash function parameters until we find a set that works
 	# for these keys.  The multipliers are chosen to be primes that are cheap
-	# to calculate via shift-and-add, so don't change them without care.
+	# to calculate via lea or shift, so don't change them without care.
+	# The list of multipliers is designed such that by iterating through
+	# adjacent pairs, we first try a couple combinations where at least one
+	# multiplier is compiled to a single lea (3 or 5). Also, we eventually
+	# want to include the largest numbers in our search for the sake of
+	# finicky key sets such as a large number of unicode codepoints.
 	# (Commonly, random seeds are tried, but we want reproducible results
 	# from this program so we don't do that.)
-	my $hash_mult1 = 31;
+	my @HASH_MULTS = (3, 5, 17, 31, 127, 257, 8191);
+	my $hash_mult1;
 	my $hash_mult2;
 	my $hash_seed1;
 	my $hash_seed2;
 	my @subresult;
   FIND_PARAMS:
-	foreach (127, 257, 521, 1033, 2053)
+	foreach my $i (0 .. $#HASH_MULTS - 1)
 	{
-		$hash_mult2 = $_;    # "foreach $hash_mult2" doesn't work
+		$hash_mult1 = $HASH_MULTS[$i];
+		$hash_mult2 = $HASH_MULTS[$i+1];
 		for ($hash_seed1 = 0; $hash_seed1 < 10; $hash_seed1++)
 		{
 			for ($hash_seed2 = 0; $hash_seed2 < 10; $hash_seed2++)
